package com.beechwood.certificates;

import java.util.Hashtable;

public class ASN1Constants {

  public static final int ID_BOOLEAN = 0x01;
  public static final int ID_INTEGER = 0x02;
  public static final int ID_BIT_STRING = 0x03;
  public static final int ID_OCTET_STRING = 0x04;
  public static final int ID_NULL = 0x05;
  public static final int ID_OBJECT_IDENTIFIER = 0x06;
  public static final int ID_OBJECT_DESCRIPTOR = 0x07;
  public static final int ID_EXTERNAL = 0x08;
  public static final int ID_REAL = 0x09;
  public static final int ID_ENUMERATED = 0x0A;
  public static final int ID_SEQUENCE = 0x10;
  public static final int ID_SET = 0x11;
  public static final int ID_NUMERIC_STRING = 0x12;
  public static final int ID_PRINTABLE_STRING = 0x13;
  public static final int ID_TELETEX_STRING = 0x14;
  public static final int ID_VIDEOTEX_STRING = 0x15;
  public static final int ID_IA5_STRING = 0x16;
  public static final int ID_UTC_TIME = 0x17;
  public static final int ID_GENERALIZED_TIME = 0x18;
  public static final int ID_GRAPHIC_STRING = 0x19;
  public static final int ID_VISIBLE_STRING = 0x1A;
  public static final int ID_GENERAL_STRING = 0x1B;
  public static final int ID_CHARACTER_STRING = 0x1C;

  public static final int MASK_CLASS = 0xC0;
  public static final int BIT_6 = 0x20;
  public static final int BITS_5_1 = 0x1F;
  public static final int BIT_8 = 0x80;
  public static final int BITS_7_1 = 0x7F;
  public static final int BITS_8_1 = 0xFF;

  public static final int CLASS_CONTEXT_SPECIFIC = 0x80;

  public static Hashtable oidMap = initOIDMap();

  private static Hashtable initOIDMap() {
    String[][] oidValues =  {
      {"1.2.840.10040.4.1", "id-dsa"},
      {"1.2.840.10040.4.3", "id-dsa-with-sha1"},
      {"1.2.840.113533.7.65.0", "entrustVersion"},
      {"1.2.840.113549.1.1.1", "rsaEncryption"},
      {"1.2.840.113549.1.1.2", "md2WithRSAEncryption"},
      {"1.2.840.113549.1.1.4", "md5WithRSAEncryption"},
      {"1.2.840.113549.1.1.5", "sha-1WithRSAEncryption"},
      {"1.2.840.113549.1.9.1", "e-mailAddress"},
      {"1.3.14.3.2.12", "dsa"},
      {"1.3.14.3.2.27", "dsa With SHA1"},
      {"2.16.840.1.113730.1.1", "netscapeCertificateType"},
      {"2.16.840.1.113730.1.12", "SslServerName"},
      {"2.16.840.1.113730.1.13", "netscapeCertificateComment"},
      {"2.16.840.1.113730.1.2", "BaseURL"},
      {"2.16.840.1.113730.1.3", "RevocationURL"},
      {"2.16.840.1.113730.1.4", "CARevocationURL"},
      {"2.16.840.1.113730.1.7", "renewalURL"},
      {"2.16.840.1.113730.1.8", "netscapeCaPolicyURL"},
      {"2.5.29.14", "subjectKeyIdentifier"},
      {"2.5.29.15", "keyUsage"},
      {"2.5.29.16", "privateKeyUsagePeriod"},
      {"2.5.29.17", "subjectAlternativeName"},
      {"2.5.29.18", "issuerAlternativeName"},
      {"2.5.29.19", "basicConstraints"},
      {"2.5.29.20", "cRLNumber"},
      {"2.5.29.21", "reasonCode"},
      {"2.5.29.23", "holdInstructionCode"},
      {"2.5.29.24", "invalidityDate"},
      {"2.5.29.27", "deltaCRLIndicator"},
      {"2.5.29.28", "issuingDistributionPoint"},
      {"2.5.29.29", "certificateIssuer"},
      {"2.5.29.3", "certificate policies - old"},
      {"2.5.29.30", "nameConstraints"},
      {"2.5.29.31", "cRLDistributionPoints"},
      {"2.5.29.32", "certificatePolicies"},
      {"2.5.29.33", "policyMappings"},
      {"2.5.29.35", "authorityKeyIdentifier"},
      {"2.5.29.36", "policyConstraints"},
      {"2.5.4.10", "O"},
      {"2.5.4.11", "OU"},
      {"2.5.4.13", "??"},
      {"2.5.4.3", "CN"},
      {"2.5.4.6", "C"},
      {"2.5.4.7", "L"},
      {"2.5.4.8", "S"},
      {"2.5.8.1.1", "rsa"}
    };
    Hashtable ht = new Hashtable();
    for (int i = 0; i < oidValues.length; ++i) {
      ht.put(oidValues[i][0], oidValues[i][1]);
    }
    return ht;
  }
}
