package com.beechwood.certificates;

import java.io.*;

public class X509Certificate {

  private byte[] encodedCertificate;
  private ASN1Value certificateSequence;
  private byte[] tbsSequence;
  private byte[] derCertificate;
  private X509TBSCertificate tbsCertificate;
  private X509AlgorithmIdentifier signatureAlgorithm;
  private ASN1Value signature;

  public X509Certificate(String fileName)
    throws X509CertificateException {
    derCertificate = readCertificate(fileName);
    parse();
  }

  private byte[] readCertificate(String fileName)
    throws X509CertificateException {
    boolean inPEM = false;
    BufferedReader br = null;
    try {
      br = new BufferedReader(new FileReader(fileName));
    }
    catch (FileNotFoundException e) {
      throw new X509CertificateException("file not found");
    }
    ByteArrayOutputStream rawCertificate =
      new ByteArrayOutputStream();
    while(true) {
      try {
        String line = br.readLine();
        if (line == null)
          break;
        if (inPEM) {
          if ((line.startsWith("-")) &&
              (line.indexOf("END") >=0) &&
              (line.indexOf("CERTIFICATE") >=0)) {
            inPEM = false;
            break;
          }
          try {
            rawCertificate.write(Base64.decode(line));
          }
          catch (Base64Exception b4e) {
            throw new X509CertificateException
              ("improper base64 encoding");
          }
        }
        if ((line.startsWith("-")) &&
            (line.indexOf("BEGIN") >=0) &&
            (line.indexOf("CERTIFICATE") >=0)) {
          inPEM = true;
        }
      }
      catch (IOException ioe) {
        throw new X509CertificateException("IOException " +
          ioe.getMessage());
      }
    }
    if (inPEM)
      throw new X509CertificateException("invalid certificate");
    return rawCertificate.toByteArray();
  }

  private void parse() throws X509CertificateException {
    byte[] ba;
    certificateSequence = new ASN1Value(derCertificate, 0);
    if (certificateSequence.hasIndefiniteLength()) {
      ba = new byte[derCertificate.length -2];
      System.arraycopy(derCertificate,2,ba,0,
        derCertificate.length - 2);
    }
    else {
      ba = new byte[certificateSequence.data().length];
      System.arraycopy(certificateSequence.data(),0,ba,0,
        certificateSequence.data().length);
    }
    int p = 0;
    tbsCertificate = new X509TBSCertificate(ba,0);
    p += tbsCertificate.totalBytes();
    ASN1Value asn1Value = new ASN1Value(ba, p);
    signatureAlgorithm = new X509AlgorithmIdentifier(asn1Value);
    p += signatureAlgorithm.totalBytes();
    signature = new ASN1Value(ba, p);
  }

  public X509TBSCertificate getTBSCertificate() {
    return tbsCertificate;
  }

  public X509AlgorithmIdentifier getSignatureAlgorithm() {
    return signatureAlgorithm;
  }

  public String getSignature() {
    return signature.dataAsHexString();  
  }
}
