package com.beechwood.certificates;

import java.util.Vector;

public class X509TBSCertificate {
  int version;
  int serialNumber;
  X509AlgorithmIdentifier signature;
  X509Name issuer;
  X509Validity validity;
  X509Name subject;
  X509SubjectPublicKeyInfo subjectPublicKeyInfo;
  Vector extensions;
  int totalBytes;
  

  public X509TBSCertificate(byte[] ba, int startPosition) 
      throws X509CertificateException {
    ASN1Value tbsSequence = new ASN1Value(ba,startPosition);
    totalBytes = tbsSequence.totalBytes();
    int p = startPosition;
    ASN1Value b = new ASN1Value(tbsSequence.data(), 0);
    if ((b.getASN1Class() == ASN1Constants.CLASS_CONTEXT_SPECIFIC) && 
        (b.isConstructed()) && (b.tag() == 0)) {
      p += b.totalBytes();
      b = new ASN1Value(b.data(),0);
      try {
        version = b.getInteger() + 1;
      }
      catch (ASN1ValueException e) {
        throw new X509CertificateException("version not INTEGER");
      }
      b = new ASN1Value(tbsSequence.data(),p);
    }
    else
      version = 1;
    try {
      serialNumber = b.getInteger();
    }
    catch (ASN1ValueException e) {
      throw new X509CertificateException("version not INTEGER");
    }
    Vector v = new Vector();
    p += b.totalBytes();
    while (p < tbsSequence.length()) {
      b = new ASN1Value(tbsSequence.data(), p);
      p += b.totalBytes();
      v.addElement(b);
    }
    signature = new X509AlgorithmIdentifier((ASN1Value)v.elementAt(0));
    issuer = new X509Name((ASN1Value)v.elementAt(1));
    validity = new X509Validity((ASN1Value)v.elementAt(2));
    subject = new X509Name((ASN1Value)v.elementAt(3));
    subjectPublicKeyInfo = 
      new X509SubjectPublicKeyInfo((ASN1Value)v.elementAt(4));
    extensions = new Vector();
    for (int i = 5; i < v.size(); ++i) {
      b = (ASN1Value)(v.elementAt(i));
      if ((b.getASN1Class() == ASN1Constants.CLASS_CONTEXT_SPECIFIC) && 
          (b.tag() == 3)) {
        parseExtensions(new ASN1Value(b.data(), 0));
      }
    }
  }

  private void parseExtensions(ASN1Value asn1Value) 
      throws X509CertificateException {
    int p = 0;
    while (p < asn1Value.data().length) {
      ASN1Value obj = new ASN1Value(asn1Value.data(),p);
      extensions.addElement(new X509Extension(obj));
      p += obj.totalBytes();
    }
  }

  public X509Extension[] getExtensions() {
    //if (extensions == null) 
    //  return new X509Extension[0];
    if (extensions.size() == 0)
      return new X509Extension[0];;
    X509Extension[] e = new X509Extension[extensions.size()];
    e = (X509Extension[])(extensions.toArray(e));
    return e;
  }

  public int totalBytes() {
    return totalBytes;
  }

  public int version()  {
    return version;
  }

  public int serialNumber()  {
    return serialNumber;
  }

  public X509Name issuer() {
    return issuer;
  }

  public X509Name subject() {
    return subject;
  }

  public X509SubjectPublicKeyInfo subjectPublicKeyInfo() {
    return subjectPublicKeyInfo;
  }

  public String subjectPublicKey() {
    return subjectPublicKeyInfo.subjectPublicKey.dataAsHexString();   
  }

  public X509Validity validity() {
    return validity;
  }
}
