package com.beechwood.certificates;

import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.GregorianCalendar;

public class X509Validity {
  ASN1Value notBefore;
  ASN1Value notAfter;

  public X509Validity(ASN1Value asn1Value) {
    notBefore = new ASN1Value(asn1Value.data(), 0);
    notAfter = new ASN1Value(asn1Value.data(), notBefore.totalBytes());
  }

  public String notAfter() {
    return notAfter.getString();
  }

  private Date dateFromString(String s) 
      throws X509CertificateException {
    int len = s.length() - 1;
    if ((len != 12) && (len != 14))
      throw new X509CertificateException("invalid date");
    String pattern = new String((len == 12) ?
      "yyMMddHHmmss" : "yyyyMMddHHmmss");
    SimpleDateFormat formatter = new SimpleDateFormat(pattern);
    Date date = formatter.parse(s.substring(0,s.length()-1),// + "EST", 
      new ParsePosition(0));
    return date;
  }

  public Date notAfterDate() throws X509CertificateException {
    return dateFromString(notAfter());
  }

  public String notAfterDateString() throws X509CertificateException {
    GregorianCalendar cal = new GregorianCalendar();
    cal.setTime(dateFromString(notAfter()));
    SimpleDateFormat formatter = new SimpleDateFormat("dd MMM yyyy HH:mm:ss");
    return formatter.format(cal.getTime());
  }

  public String notBefore() {
    return notBefore.getString();
  }

  public Date notBeforeDate() throws X509CertificateException {
    return dateFromString(notBefore());
  }

  public String notBeforeDateString() throws X509CertificateException {
    GregorianCalendar cal = new GregorianCalendar();
    cal.setTime(dateFromString(notBefore()));
    SimpleDateFormat formatter = new SimpleDateFormat("dd MMM yyyy HH:mm:ss");
    return formatter.format(cal.getTime());
  }
}
